@@ -27,10 +27,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#ifdef HAVE_CLUTTER
-#include <clutter/clutter.h>
-#endif
-
 #include "ar-debug.h"
 #include "ar-runtime.h"
 #include "ar-string-utils.h"
@@ -251,16 +247,7 @@ clear_slots (AisleriotGame *game,
   for (i = 0; i < n_slots; ++i) {
     ArSlot *slot = game->slots->pdata[i];
 
-#ifdef HAVE_CLUTTER
-    if (slot->slot_renderer) {
-      clutter_actor_destroy (slot->slot_renderer);
-      g_object_unref (slot->slot_renderer);
-    }
-
-    g_byte_array_free (slot->old_cards, TRUE);
-#else
     g_ptr_array_free (slot->card_images, TRUE);
-#endif /* HAVE_CLUTTER */
     g_byte_array_free (slot->cards, TRUE);
 
     g_slice_free (ArSlot, slot);
@@ -323,7 +310,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
 
   port = scm_open_output_string ();
   scm_display (throw_args, port);
-  string = scm_to_locale_string (scm_get_output_string (port));
+  string = scm_to_utf8_string (scm_get_output_string (port));
   scm_dynwind_free (string);
   scm_close_output_port (port);
   g_string_append (message, string);
@@ -331,7 +318,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
   port = scm_open_output_string ();
   g_string_append (message, "\nScheme tag:\n\t");
   scm_display (tag, port);
-  string = scm_to_locale_string (scm_get_output_string (port));
+  string = scm_to_utf8_string (scm_get_output_string (port));
   scm_dynwind_free (string);
   scm_close_output_port (port);
   g_string_append (message, string);
@@ -341,7 +328,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
   if (!scm_is_false (stack)) {
     port = scm_open_output_string ();
     scm_display_backtrace (stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
-    string = scm_to_locale_string (scm_get_output_string (port));
+    string = scm_to_utf8_string (scm_get_output_string (port));
     scm_dynwind_free (string);
     scm_close_output_port (port);
     g_string_append (message, string);
@@ -603,7 +590,7 @@ cscmi_add_slot (SCM slot_data)
 
   if (game->state > GAME_BEGIN) {
     return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
-                      scm_list_1 (scm_from_locale_string ("Cannot add a new slot after the game has started.")));
+                      scm_list_1 (scm_from_utf8_string ("Cannot add a new slot after the game has started.")));
   }
 
 #define EQUALS_SYMBOL(string,object) (scm_is_true (scm_equal_p (scm_from_locale_symbol (string), object)))
@@ -626,7 +613,9 @@ cscmi_add_slot (SCM slot_data)
   /* 3rd argument is the slot type (optionally) */
   slot_type = SCM_CDDDR (slot_data);
   if (slot_type != SCM_EOL) {
-    if (EQUALS_SYMBOL ("foundation", SCM_CAR (slot_type))) {
+    if (EQUALS_SYMBOL ("chooser", SCM_CAR (slot_type))) {
+      type = AR_SLOT_CHOOSER;
+    } else if (EQUALS_SYMBOL ("foundation", SCM_CAR (slot_type))) {
       type = AR_SLOT_FOUNDATION;
     } else if (EQUALS_SYMBOL ("reserve", SCM_CAR (slot_type))) {
       type = AR_SLOT_RESERVE;
@@ -643,7 +632,7 @@ cscmi_add_slot (SCM slot_data)
 
 #ifdef GNOME_ENABLE_DEBUG
   _AR_DEBUG_IF (AR_DEBUG_SCHEME) {
-    static const char *types[] = { "unknown", "foundation", "reserve", "stock", "tableau", "waste" };
+    static const char *types[] = { "unknown", "chooser", "foundation", "reserve", "stock", "tableau", "waste" };
 
     ar_debug_print (AR_DEBUG_SCHEME,
                         "Adding new slot %d type %s\n",
@@ -670,11 +659,7 @@ cscmi_add_slot (SCM slot_data)
   slot->expanded_down = expanded_down != FALSE;
   slot->expanded_right = expanded_right != FALSE;
 
-#ifdef HAVE_CLUTTER
-  slot->old_cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#else
   slot->card_images = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#endif
 
   slot->needs_update = TRUE;
 
@@ -697,7 +682,7 @@ scm_gettext (SCM message)
 
   scm_dynwind_begin (0);
 
-  input = scm_to_locale_string (message);
+  input = scm_to_utf8_string (message);
   scm_dynwind_free (input);
   if (!input)
     goto out;
@@ -705,7 +690,7 @@ scm_gettext (SCM message)
   output = g_dgettext (NULL, input);
 
   if (input != output) {
-    translated = scm_from_locale_string (output);
+    translated = scm_from_utf8_string (output);
   } else {
     translated = message;
   }
@@ -785,7 +770,7 @@ scm_set_statusbar_message (SCM message)
 
   scm_dynwind_begin (0);
 
-  str = scm_to_locale_string (message);
+  str = scm_to_utf8_string (message);
   scm_dynwind_free (str);
   if (!str)
     goto out;
@@ -952,7 +937,7 @@ scm_set_lambda_x (SCM symbol,
   }
 
   return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
-                    scm_list_1 (scm_from_locale_string ("Unknown lambda name in set-lambda!")));
+                    scm_list_1 (scm_from_utf8_string ("Unknown lambda name in set-lambda!")));
 }
 
 static SCM
@@ -980,7 +965,7 @@ scm_update_score (SCM new_score)
   AisleriotGame *game = app_game;
   char *score;
 
-  score = scm_to_locale_string (new_score);
+  score = scm_to_utf8_string (new_score);
   if (g_strcmp0 (score, game->score) != 0) {
     free (game->score);
     game->score = score;
@@ -1052,7 +1037,7 @@ scm_delayed_call (SCM callback)
   /* We can only have one pending delayed call! */
   if (game->delayed_call_timeout_id != 0) {
     return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
-                      scm_list_1 (scm_from_locale_string ("Already have a delayed callback pending.")));
+                      scm_list_1 (scm_from_utf8_string ("Already have a delayed callback pending.")));
   }
 
   /* We need to protect the callback data from being GC'd until the
@@ -1386,7 +1371,7 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
                            NULL);
   variable = scm_c_module_lookup (scm_the_root_module (), "%load-path");
   scm_variable_set_x (variable, scm_append_x (scm_list_2 (scm_variable_ref (variable),
-                                                          scm_list_1 (scm_from_locale_string (path)))));
+                                                          scm_list_1 (scm_from_utf8_string (path)))));
   g_free (path);
 
   path = g_build_filename (ar_runtime_get_directory (AR_RUNTIME_PKG_LIBRARY_DIRECTORY),
@@ -1395,7 +1380,7 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
                            NULL);
   variable = scm_c_module_lookup (scm_the_root_module (), "%load-compiled-path");
   scm_variable_set_x (variable, scm_append_x (scm_list_2 (scm_variable_ref (variable),
-                                                          scm_list_1 (scm_from_locale_string (path)))));
+                                                          scm_list_1 (scm_from_utf8_string (path)))));
   g_free (path);
 }
 
@@ -1432,6 +1417,10 @@ ar_slot_get_type_string (ArSlot *slot)
     case AR_SLOT_UNKNOWN:
       text = NULL;
       break;
+    case AR_SLOT_CHOOSER:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "chooser");
+      break;
     case AR_SLOT_FOUNDATION:
       /* Translators: this is the name of a type of card slot */
       text = C_("slot type", "foundation");
@@ -1480,6 +1469,10 @@ ar_slot_get_hint_string (ArSlot *slot,
   card_name = ar_card_get_locale_name (CARD (slot->cards->data[cardid]));
 
   switch (slot->type) {
+    case AR_SLOT_CHOOSER:
+      /* Translators: %s is the name of the card; "chooser" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on chooser"), card_name);
+
     case AR_SLOT_FOUNDATION:
       /* Translators: %s is the name of the card; "foundation" is the name of a type of card slot */
       return g_strdup_printf (C_("slot hint", "%s on foundation"), card_name);
@@ -1682,12 +1675,12 @@ game_scm_load_game (void *user_data)
 
   scm_dynwind_begin (0);
 
-  scm_primitive_load_path (scm_from_locale_string (game_module));
+  scm_primitive_load_path (scm_from_utf8_string (game_module));
 
   for (i = 0; i <= LAST_MANDATORY_LAMBDA; ++i) {
     if (scm_is_false (scm_procedure_p (game->lambdas[i]))) {
       scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-                 scm_list_3 (scm_from_locale_string ("Not a procedure"),
+                 scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                              scm_from_int (i),
                              game->lambdas[i]));
     }
@@ -1695,13 +1688,13 @@ game_scm_load_game (void *user_data)
   if ((game->features & FEATURE_DROPPABLE) &&
       scm_is_false (scm_procedure_p (game->lambdas[DROPPABLE_LAMBDA])))
     scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-               scm_list_3 (scm_from_locale_string ("Not a procedure"),
+               scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                            scm_from_int (DROPPABLE_LAMBDA),
                            game->lambdas[i]));
   if ((game->features & FEATURE_DEALABLE) &&
       scm_is_false (scm_procedure_p (game->lambdas[DEALABLE_LAMBDA])))
     scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-               scm_list_3 (scm_from_locale_string ("Not a procedure"),
+               scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                            scm_from_int (DEALABLE_LAMBDA),
                            game->lambdas[i]));
 
@@ -2174,7 +2167,7 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
@@ -2189,12 +2182,12 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1) || !scm_is_string (string2))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
 
-      str2 = scm_to_locale_string (string2);
+      str2 = scm_to_utf8_string (string2);
       scm_dynwind_free (str2);
       if (!str2)
         break;
@@ -2214,11 +2207,11 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1) || !scm_is_string (string2))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
-      str2 = scm_to_locale_string (string2);
+      str2 = scm_to_utf8_string (string2);
       scm_dynwind_free (str2);
       if (!str2)
         break;
@@ -2314,7 +2307,7 @@ aisleriot_game_get_options (AisleriotGame *game)
       if (!scm_is_string (entryname))
         continue; /* Shouldn't happen */
 
-      entrynamestr = scm_to_locale_string (entryname);
+      entrynamestr = scm_to_utf8_string (entryname);
       scm_dynwind_free (entrynamestr);
       if (!entrynamestr)
         continue;
@@ -2572,44 +2565,6 @@ aisleriot_game_get_score (AisleriotGame *game)
   return game->score ? game->score : "";
 }
 
-#ifdef HAVE_CLUTTER
-
-void
-aisleriot_game_get_card_offset (ArSlot *slot,
-                                guint card_num,
-                                gboolean old_cards,
-                                gint *xoff, gint *yoff)
-{
-  gint n_cards, exposed;
-
-  if (old_cards) {
-    n_cards = (gint) slot->old_cards->len;
-    exposed = (gint) slot->old_exposed;
-  } else {
-    n_cards = (gint) slot->cards->len;
-    exposed = (gint) slot->exposed;
-  }
-
-  if (card_num >= n_cards - exposed) {
-    gint idx = card_num + exposed - n_cards;
-    *xoff = slot->pixeldx * idx;
-    *yoff = slot->pixeldy * idx;
-  } else {
-    *xoff = 0;
-    *yoff = 0;
-  }
-}
-
-void
-aisleriot_game_reset_old_cards (ArSlot *slot)
-{
-  g_byte_array_set_size (slot->old_cards, 0);
-  g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
-  slot->old_exposed = slot->exposed;
-}
-
-#endif /* HAVE_CLUTTER */
-
 static void
 append_games_from_path (GHashTable *hash_table,
                         const char *path,
